import numpy as np
from itertools import product

def shift_xyz_data(data, shift):
	res = []
	for elem,x,y,z in data:
		res.append( [elem] + list(np.round(np.array([x,y,z]) + np.array(shift), 6)))
	return res
	
def multiply_xyz_data(data, pbcell, multiplier, mulmul):
	new_data = []
	for i in  product(*[range(i) for i in multiplier]):
		shifts = [v*np.array([float(n)]*3) for n,v in zip(i, pbcell.cell.cell_basis)]
		shift = sum(shifts)*mulmul	# little gap b/w images
		new_data += shift_xyz_data(data, shift)
	return new_data
	
def generate_xyz(data, comment='', pbcell = None, multiplier = [1,1,1], mulmul=1.0):

	if pbcell is not None:
		data = [ [e] + list(pbcell.wrap_in_cell_orhonorm([x,y,z], precission=3)) for e,x,y,z in data ]
		data = multiply_xyz_data(data[:], pbcell, multiplier, mulmul)

	#res = str(len(data)) + '\n' + comment + "\n"
	res = ""
	N = 0
	for row in data:
		#print 'generate_xyz', row
		if row[0] != 'XX':
			res += '\t'.join([str(x) for x in row]) + '\n';
			N +=1
			
	return str(N) + '\n' + comment + "\n" + res

def oneline_xyz(data):		
	return '; '.join( ['(' + ', '.join([str(x) for x in row]) + ')' for row in data])


'''	
def write_xyz(cells, regr, unique_lcs, filename, comment, marks):
	regr.mark_by_cells(cells, unique_lcs, marks)
	
	output = open(filename, 'w')
	output.write(generate_xyz(regr.generate_xyz_data(hide=[]), comment=comment))
'''

def shifted_xyz_mark(marks, idx, lctype, coords):
	m = marks[ (idx,lctype) ]
	if isinstance(m, basestring):
		return [ m ] + list(coords)
	else:
		return [ m[0] ] + list(coords + m[1])

"""
def xyz_for_lc_state(lc_state, cl, atoms, marks):

	incell_shifts = [cl.to_orthonorm_basis(c) for _,c in atoms]
	
	data = []
	for lc,shift in zip(lc_state, incell_shifts):
		cc_dict = lc.to_cart_config_dict(shift)
		data += [shifted_xyz_mark(marks, idx,lc.lctype,coords) for coords,idx in cc_dict.iteritems()]
		
	return data[:]
"""

def get_lc_xyz_data(lc, marks, shift=tuple([0., 0., 0.])):
	cc_dict = lc.to_cart_config_dict(shift)
	data = [ [marks[idx] ] + list(np.round(coords, 6) ) for coords,idx in cc_dict.iteritems()]
	return data[:]

	


def gen_xyz_data(cells, regr, lc_marker, pbc_wrapper, with_lattice):
	data = []
	for cell_i,cell in enumerate(cells):
		if any(cell):		
			for lci,x in enumerate(cell):
				if (x):
					zero_coords = regr[cell_i].coords
					unwrapped_data = lc_marker(lci)
					data += [ [row[0]] + list( pbc_wrapper( zero_coords + row[1:] ) ) for row in unwrapped_data]
		else:
			if with_lattice:
				data.append( ['H'] + list(regr[cell_i].coords) ) 
	return data
	
def write_xyz_2(cells, regr, lc_marker, filename, comment, pbc_wrapper, with_lattice=True, background = []):
	output = open(filename, 'w')
	data = background + gen_xyz_data( cells, regr, lc_marker, pbc_wrapper, with_lattice)
	output.write( generate_xyz(data, comment=comment ))
	return data

def cells_as_xyz_data(regr, cells, rn_states):
	assert (cells is not None)
	xyz_data = []
	for cell, vertex in zip(cells, regr.vertex_index):
		state = rn_states[cell]
		c0 = vertex.coords
		xyz = state.get_prop('xyz', 1)
		xyz_data += shift_xyz_data(xyz, c0)
	return xyz_data
	
def save_cells_as_xyz(fn, regr, cells, rn_states, pbcell = None, comment='', multiplier = [1,1,1], mulmul=1.0):
	if cells is not None:
		open(fn,'a').write(generate_xyz(cells_as_xyz_data(regr, cells, rn_states), comment, pbcell, multiplier, mulmul) )
	

